bluefin-0.6.0.0: The Bluefin effect system
Safe HaskellSafe-Inferred
LanguageHaskell2010

Bluefin.Capability

Synopsis

    Historical commentary

    Bluefin is in a transitionary phase moving away from the old terminology of "handle" and naming handles/effects based on MTL/transformers style names (e.g. Exception/Reader/Stream) and moving towards calling these things "capabilities" and naming them after their main operation (e.g. Throw/Ask/Yield). You are encouraged to use the API beneath Bluefin.Capability because that will be the supported API in the future.

    You are encouraged to change your usage of the old modules on the left to the new modules on the right:

    OldNew
    Bluefin.ReaderBluefin.Capability.Ask
    Bluefin.HandleReaderBluefin.Capability.AskCapability
    Bluefin.ConsumeBluefin.Capability.Await
    Bluefin.JumpBluefin.Capability.JumpTo
    Bluefin.StateBluefin.Capability.Modify
    Bluefin.CoroutineBluefin.Capability.Request
    Bluefin.EarlyReturnBluefin.Capability.ReturnEarly
    Bluefin.WriterBluefin.Capability.Tell
    Bluefin.ExceptionBluefin.Capability.Throw
    Bluefin.StreamBluefin.Capability.Yield