| Safe Haskell | Safe-Inferred |
|---|---|
| Language | Haskell2010 |
Bluefin.Capability
Contents
Synopsis
Historical commentary
Bluefin is in a transitionary phase moving away from the old
terminology of "handle" and naming handles/effects based on
MTL/transformers style names
(e.g. Exception/Reader/Stream) and moving towards
calling these things "capabilities" and naming them after their
main operation (e.g. Throw/Ask/Yield). You are
encouraged to use the API beneath Bluefin.Capability because
that will be the supported API in the future.
You are encouraged to change your usage of the old modules on the left to the new modules on the right: